/*---------------------------------------------------------------------------

  FILENAME:
        radlist.c

  PURPOSE:
        Provide the linked list utilities.

  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        5/26/95         M.S. Teel       0               Original
        9/26/99         MS Teel         1               Port to pSOS
        3/22/01         M.S. Teel       2               Port to Linux

  NOTES:
        

  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*      ... OS include files
*/
#include <stdio.h>
#include <stdlib.h>

/*      ... System include files
*/

/*      ... Library include files
*/

/*      ... Local include files
*/
#include "radlist.h"


/*      ... global memory declarations
*/

/*      ... global memory referenced
*/

/*      ... static (local) memory declarations
*/



/*  ... Allocate and initialize a list to empty state
*/
LIST_ID radListCreate (void)
    {
    LIST_ID     list;


    if ((list = (LIST_ID) malloc (sizeof (*list))) != NULL)
        {
        list->firstNode = &list->dummyFirst;
        list->lastNode = &list->dummyLast;
        list->firstNode->prevNode = NULL;
        list->firstNode->nextNode = list->lastNode;
        list->lastNode->prevNode = list->firstNode;
        list->lastNode->nextNode = NULL;
        list->noNodes = 0;
        }

    return (list);
    }

LIST_ID radListReset
    (
    LIST_ID     list
    )
    {
    list->firstNode = &list->dummyFirst;
    list->lastNode = &list->dummyLast;
    list->firstNode->prevNode = NULL;
    list->firstNode->nextNode = list->lastNode;
    list->lastNode->prevNode = list->firstNode;
    list->lastNode->nextNode = NULL;
    list->noNodes = 0;

    return (list);
    }



void radListDelete
    (
    LIST_ID     list
    )
    {
    free (list);
    return;
    }


void radListInsertAfter
    (
    LIST_ID     list,
    NODE_PTR    afterThisNode,
    NODE_PTR    node
    )
    {
    node->nextNode = afterThisNode->nextNode;
    node->prevNode = afterThisNode;
    afterThisNode->nextNode = node;
    node->nextNode->prevNode = node;
    list->noNodes += 1;
    return;
    }


void radListInsertBefore
    (
    LIST_ID     list,
    NODE_PTR    beforeThisNode,
    NODE_PTR    node
    )
    {

    node->prevNode = beforeThisNode->prevNode;
    node->nextNode = beforeThisNode;
    beforeThisNode->prevNode = node;
    node->prevNode->nextNode = node;
    list->noNodes += 1;
    return;
    }


void radListAddToFront
    (
    LIST_ID     list,
    NODE_PTR    node
    )
    {
    node->prevNode = list->firstNode;
    node->nextNode = list->firstNode->nextNode;
    list->firstNode->nextNode = node;
    node->nextNode->prevNode = node;
    list->noNodes += 1;
    return;
    }


void radListAddToEnd
    (
    LIST_ID     list,
    NODE_PTR    node
    )
    {
    node->prevNode = list->lastNode->prevNode;
    node->nextNode = list->lastNode;
    list->lastNode->prevNode = node;
    node->prevNode->nextNode = node;
    list->noNodes += 1;
    return;
    }


void radListRemove
    (
    LIST_ID     list,
    NODE_PTR    node
    )
    {
    node->nextNode->prevNode = node->prevNode;
    node->prevNode->nextNode = node->nextNode;
    list->noNodes -= 1;
    return;
    }


NODE_PTR radListRemoveFirst
    (
    LIST_ID     list
    )
    {
    NODE_PTR    node;


    if (list->firstNode->nextNode == list->lastNode)
        return (NULL);

    node = list->firstNode->nextNode;
    list->firstNode->nextNode = node->nextNode;
    node->nextNode->prevNode = list->firstNode;
    list->noNodes -= 1;
    return (node);
    }


NODE_PTR radListRemoveLast
    (
    LIST_ID     list
    )
    {
    NODE_PTR    node;


    if (list->lastNode->prevNode == list->firstNode)
        return (NULL);

    node = list->lastNode->prevNode;
    list->lastNode->prevNode = node->prevNode;
    node->prevNode->nextNode = list->lastNode;
    list->noNodes -= 1;
    return (node);
    }


int radListGetNumberOfNodes
    (
    LIST_ID     list
    )
    {
    return (list->noNodes);
    }


NODE_PTR radListGetFirst
    (
    LIST_ID     list
    )
    {
    if (list->firstNode->nextNode == list->lastNode)
        return (NULL);

    return (list->firstNode->nextNode);
    }


NODE_PTR radListGetLast
    (
    LIST_ID     list
    )
    {
    if (list->lastNode->prevNode == list->firstNode)
        return (NULL);
    return (list->lastNode->prevNode);
    }


NODE_PTR radListGetNext
    (
    LIST_ID     list,
    NODE_PTR    afterThisNode           /* NULL gets first one  */
    )
    {
    if (afterThisNode == NULL)
        return (radListGetFirst (list));

    if (afterThisNode->nextNode == list->lastNode)
        return (NULL);
    else
        return (afterThisNode->nextNode);
    }


NODE_PTR radListGetPrevious
    (
    LIST_ID     list,
    NODE_PTR    beforeThisNode          /* NULL gets first one  */
    )
    {
    if (beforeThisNode == NULL)
        return (radListGetLast (list));

    if (beforeThisNode->prevNode == list->firstNode)
        return (NULL);
    else
        return (beforeThisNode->prevNode);
    }

#if 0
void listDebug
    (
    LIST_ID list
    )
    {
    printf ("dfirst=%x,dlast=%x,first=%x,last=%x,dummyFirst=%x,dummyLast=%x\n",
        list->firstNode,
        list->lastNode,
        list->firstNode->nextNode,
        list->lastNode->prevNode,
        &list->dummyFirst,
        &list->dummyLast);
    }
#endif
