/*---------------------------------------------------------------------------

  FILENAME:
        session.h

  PURPOSE:
        Provide session processing for the qmail-scanner log analysis tool.

  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        05/29/04        M.S. Teel       0               Original

  NOTES:

  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/
#ifndef INC_qmssessionh
#define INC_qmssessionh

/*  ... System include files
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>

/*  ... Library include files
*/

/*  ... Local include files
*/
#include "radlist.h"


#define MAX_DEST_DOMAINS        64


/*  ... data structures for session processing
*/
typedef enum
{
    ACCT_DELIVER        = 0,
    ACCT_RELAY,
    ACCT_VIRUS_IN,
    ACCT_VIRUS_OUT,
    ACCT_SP_MARK,
    ACCT_SP_DELETE,
    ACCT_SP_REJECT,
    ACCT_SP_QUARANTINE,
    ACCT_SP_SCORE,
    ACCT_SP_COUNT,
    ACCT_TIME_TOTAL,
    ACCT_MAX
} ACCT_STAT_TYPES;

typedef struct
{
    char        name[256];
    char        domain[128];
} QMS_DOMAIN;

typedef struct
{
    NODE        node;
    int         procId;
    char        accountName[256];
    int         incoming;
    double      stats[ACCT_MAX];
} QMS_SESSION;

typedef struct
{
    NODE        node;
    char        accountName[256];
    double      stats[ACCT_MAX];
} QMS_ACCOUNT;

typedef struct
{
    NODE        node;
    char        description[256];
    int         count;
} QMS_VIRUS_DATA;

typedef struct
{
    LIST        sessionList;
    LIST        accountList;
    LIST        virusList;
    int         sortKey;
} QMS_SESSION_MGR;


/*  ... function prototypes
*/
extern int qmsSessionInit (int sortKey);

extern void qmsSessionExit (void);

extern void qmsSessionProcess (char *tokens[]);

extern void qmsSessionPrintResults (void);


#endif
